.tip-skyblue {
	z-index:1000;
	text-align:left;
	padding:8px 10px;
	min-width:50px;
	max-width:300px;
	color:#37356c;
	background-color:#b9f3f8;
	background-image:url(tip-skyblue.png); /* bgImageFrameSize == 9 works fine */
	/**
	 * - If you set a background-image, border/padding/background-color will be ingnored.
	 *   You can set any padding to .tip-inner instead if you need.
	 * - If you want a tiled background-image and border/padding for the tip,
	 *   set the background-image to .tip-inner instead.
	 */
}
.tip-skyblue .tip-inner {
	font:bold 13px/15px 'trebuchet ms',arial,helvetica,sans-serif;
	padding:0 3px 1px 3px;
}

/* Configure an arrow image - the script will automatically position it on the correct side of the tip */
.tip-skyblue .tip-arrow-top {
	margin-top:-12px;
	margin-left:0;
	top:0;
	left:0;
	width:11px;
	height:19px;
	background:url(tip-skyblue_arrows.png) no-repeat;
}
.tip-skyblue .tip-arrow-right {
	margin-top:0;
	margin-left:-7px;
	top:0;
	left:100%;
	width:18px;
	height:13px;
	background:url(tip-skyblue_arrows.png) no-repeat -20px 0;
}
.tip-skyblue .tip-arrow-bottom {
	margin-top:-8px;
	margin-left:0;
	top:100%;
	left:0;
	width:11px;
	height:20px;
	background:url(tip-skyblue_arrows.png) no-repeat -40px 0;
}
.tip-skyblue .tip-arrow-left {
	margin-top:0;
	margin-left:-11px;
	top:0;
	left:0;
	width:18px;
	height:13px;
	background:url(tip-skyblue_arrows.png) no-repeat -60px 0;
}