.tip-green {
	z-index:1000;
	text-align:left;
	border:1px solid #bdde98;
	padding:7px 10px;
	min-width:50px;
	max-width:300px;
	color:#860404;
	background-color:#dff6c6;
	/**
	 * - If you set a background-image, border/padding/background-color will be ingnored.
	 *   You can set any padding to .tip-inner instead if you need.
	 * - If you want a tiled background-image and border/padding for the tip,
	 *   set the background-image to .tip-inner instead.
	 */
}
.tip-green .tip-inner {
	font:bold 12px/14px arial,helvetica,sans-serif;
}

/* Configure an arrow image - the script will automatically position it on the correct side of the tip */
.tip-green .tip-arrow-top {
	margin-top:-10px;
	margin-left:6px;
	top:0;
	left:0;
	width:10px;
	height:10px;
	background:url(tip-green_arrows.gif) no-repeat;
}
.tip-green .tip-arrow-right {
	margin-top:6px;
	margin-left:0;
	top:0;
	left:100%;
	width:10px;
	height:10px;
	background:url(tip-green_arrows.gif) no-repeat -10px 0;
}
.tip-green .tip-arrow-bottom {
	margin-top:0;
	margin-left:6px;
	top:100%;
	left:0;
	width:10px;
	height:10px;
	background:url(tip-green_arrows.gif) no-repeat -20px 0;
}
.tip-green .tip-arrow-left {
	margin-top:6px;
	margin-left:-10px;
	top:0;
	left:0;
	width:10px;
	height:10px;
	background:url(tip-green_arrows.gif) no-repeat -30px 0;
}