.tip-twitter {
	opacity:0.8;
	z-index:1000;
	text-align:left;
	border-radius:4px;
	-moz-border-radius:4px;
	-webkit-border-radius:4px;
	padding:8px 8px;
	max-width:200px;
	color:#fff;
	background-color:#000;
	/**
	 * - If you set a background-image, border/padding/background-color will be ingnored.
	 *   You can set any padding to .tip-inner instead if you need.
	 * - If you want a tiled background-image and border/padding for the tip,
	 *   set the background-image to .tip-inner instead.
	 */
}
.tip-twitter .tip-inner {
	font:bold 11px/14px 'Lucida Grande',sans-serif;
}

/* Configure an arrow image - the script will automatically position it on the correct side of the tip */
.tip-twitter .tip-arrow-top {
	margin-top:-5px;
	margin-left:-5px; /* approx. half the width to center it */
	top:0;
	left:50%;
	width:9px;
	height:5px;
	background:url(tip-twitter_arrows.gif) no-repeat;
}
.tip-twitter .tip-arrow-right {
	margin-top:-4px; /* approx. half the height to center it */
	margin-left:0;
	top:50%;
	left:100%;
	width:5px;
	height:9px;
	background:url(tip-twitter_arrows.gif) no-repeat -9px 0;
}
.tip-twitter .tip-arrow-bottom {
	margin-top:0;
	margin-left:-5px; /* approx. half the width to center it */
	top:100%;
	left:50%;
	width:9px;
	height:5px;
	background:url(tip-twitter_arrows.gif) no-repeat -18px 0;
}
.tip-twitter .tip-arrow-left {
	margin-top:-4px; /* approx. half the height to center it */
	margin-left:-5px;
	top:50%;
	left:0;
	width:5px;
	height:9px;
	background:url(tip-twitter_arrows.gif) no-repeat -27px 0;
}