# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-18 12:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('my_app', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='musicgenre',
            name='history',
            field=models.TextField(blank=True, null=True),
        ),
    ]
